//
//  VNKService.m
//  VenueKit
//
//  Created by Logan Collins on 3/13/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import "VNKService.h"
#import "VNKService_Private.h"


@implementation VNKService {
    NSMutableSet *_imageRequests;
}

- (id)initWithClientIdentifier:(NSString *)clientID secret:(NSString *)clientSecret {
	self = [super init];
	if (self) {
		self.clientIdentifier = clientID;
		self.clientSecret = clientSecret;
	}
	return self;
}

- (id)init {
	@throw [NSException exceptionWithName:NSInvalidArgumentException reason:[NSString stringWithFormat:@"%@ should not be directly initialized.", [self class]] userInfo:nil];
}


#pragma mark -
#pragma mark Image Request

- (id)requestImageWithDictionary:(NSDictionary *)imageDictionary size:(NSUInteger)size completionHandler:(void (^)(NSData *, NSError *))handler {
    VNKServiceImageRequest *request = [[VNKServiceImageRequest alloc] initWithService:self imageDictionary:imageDictionary size:size];
    [_imageRequests addObject:request];
    [request startWithCompletionHandler:^(NSData *imageData, NSError *error) {
        handler(imageData, error);
        [_imageRequests removeObject:request];
    }];
    return request;
}

- (void)cancelImageRequest:(id)imageRequest {
    [imageRequest cancel];
    [_imageRequests removeObject:imageRequest];
}

@end


@implementation VNKServiceImageRequest {
    NSURLConnection *_connection;
    NSMutableData *_data;
    void (^_completionHandler)(id image, NSError *error);
    NSOperationQueue *_operationQueue;
}

- (id)initWithService:(VNKService *)service imageDictionary:(NSDictionary *)imageDictionary size:(NSUInteger)size {
    NSParameterAssert(service != nil);
    NSParameterAssert(imageDictionary != nil);
    
    self = [super init];
    if (self) {
        self.service = service;
        self.imageDictionary = imageDictionary;
        
        NSUInteger adjustedSize = size;
        if (size <= 32) {
            adjustedSize = 32;
        }
        else if (size > 32 && size <= 44) {
            adjustedSize = 44;
        }
        else if (size > 44 && size <= 64) {
            adjustedSize = 64;
        }
        else if (size > 64) {
            adjustedSize = 88;
        }
        self.size = adjustedSize;
        
        _operationQueue = [[NSOperationQueue alloc] init];
        [_operationQueue setMaxConcurrentOperationCount:1];
    }
    return self;
}

- (id)init {
	@throw [NSException exceptionWithName:NSInvalidArgumentException reason:[NSString stringWithFormat:@"%@ should not be directly initialized.", [self class]] userInfo:nil];
}

- (void)startWithCompletionHandler:(void (^)(NSData *, NSError *))handler {
    NSParameterAssert(handler != nil);
    
    if (self.running) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Request has already been started" userInfo:nil];
    }
    
    _completionHandler = handler;
	
	self.running = YES;
    
    NSUInteger size = self.size;
    NSString *iconPrefix = self.imageDictionary[@"prefix"];
    NSString *iconSuffix = self.imageDictionary[@"suffix"];
    
	NSURL *requestURL = [NSURL URLWithString:[NSString stringWithFormat:@"%@%ld%@", iconPrefix, (long)size, iconSuffix]];
	NSURLRequest *request = [[NSURLRequest alloc] initWithURL:requestURL];
	
	_connection = [[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:NO];
    [_connection setDelegateQueue:_operationQueue];
    [_connection start];
}

- (void)cancel {
    [_connection cancel];
    _completionHandler = nil;
    self.running = NO;
}


#pragma mark -
#pragma mark URL Connection delegate

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    _data = [NSMutableData data];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    [_data appendData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    _completionHandler(_data, nil);
    
    _completionHandler = nil;
    _data = nil;
    _connection = nil;
    
    self.running = NO;
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    _completionHandler(nil, error);
    
    _completionHandler = nil;
    _data = nil;
    _connection = nil;
    
    self.running = NO;
}

@end
