//
//  VNKVenueSearch.m
//  VenueKit
//
//  Created by Logan Collins on 3/13/13.
//  Copyright (c) 2013 Sunflower Softworks. All rights reserved.
//

#import "VNKVenueSearch.h"

#import "VNKVenue_Private.h"
#import "VNKService_Private.h"

#import "NSDictionary+VenueKitAdditions.h"
#import "VenueKitErrors.h"


@interface VNKVenueSearch ()

@property (nonatomic, strong, readwrite) VNKService *service;
@property (nonatomic, readwrite, getter=isSearching) BOOL searching;

@end


@implementation VNKVenueSearch {
	NSOperationQueue *_operationQueue;
}

- (id)initWithService:(VNKService *)service {
	self = [super init];
	if (self) {
		self.service = service;
		
		_operationQueue = [[NSOperationQueue alloc] init];
		[_operationQueue setMaxConcurrentOperationCount:1];
	}
	return self;
}

- (id)init {
	@throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"VNKVenueSearch must be initialized with a service." userInfo:nil];
}


#pragma mark -
#pragma mark Searching

- (void)startWithCompletionHandler:(void (^)(NSArray *, NSError *))handler {
	NSParameterAssert(handler != nil);
	
	self.searching = YES;
	
	NSMutableDictionary *parameters = [NSMutableDictionary dictionary];
	
	parameters[@"client_id"] = self.service.clientIdentifier;
	parameters[@"client_secret"] = self.service.clientSecret;
	parameters[@"v"] = VENUEKIT_API_DATEVERIFIED;
	
	if (self.maximumNumberOfResults != 0) {
		parameters[@"limit"] = [NSString stringWithFormat:@"%ld", (long)self.maximumNumberOfResults];
	}
	
	if (self.naturalLanguageQuery != nil) {
		parameters[@"query"] = self.naturalLanguageQuery;
	}
	
	if (self.location != nil) {
		parameters[@"ll"] = [NSString stringWithFormat:@"%f,%f", self.location.coordinate.latitude, self.location.coordinate.longitude];
		parameters[@"alt"] = [NSString stringWithFormat:@"%f", self.location.altitude];
		if (self.location.horizontalAccuracy > 0) {
			parameters[@"llAcc"] = [NSString stringWithFormat:@"%f", self.location.horizontalAccuracy];
		}
		if (self.location.verticalAccuracy > 0) {
			parameters[@"altAcc"] = [NSString stringWithFormat:@"%f", self.location.verticalAccuracy];
		}
	}
	
	if (self.radius > 0.0) {
		parameters[@"radius"] = [NSString stringWithFormat:@"%f", self.radius];
	}
	
	NSString *parametersString = [parameters vnk_HTTPParametersString];
	
	NSURL *requestURL = [NSURL URLWithString:[NSString stringWithFormat:@"%@venues/search?%@", VENUEKIT_API_ENDPOINT, parametersString]];
	NSURLRequest *request = [[NSURLRequest alloc] initWithURL:requestURL];
	
	[NSURLConnection sendAsynchronousRequest:request queue:_operationQueue completionHandler:^(NSURLResponse *response, NSData *responseData, NSError *error) {
		if (self.searching == NO) {
			return;
		}
		
		if (response != nil) {
			NSUInteger statusCode = [(NSHTTPURLResponse *)response statusCode];
			NSError *JSONError = nil;
			NSDictionary *result = [NSJSONSerialization JSONObjectWithData:responseData options:0 error:&JSONError];
			if (result != nil) {
				if (statusCode == 200) {
					NSArray *venueDicts = result[@"response"][@"venues"];
					NSMutableArray *venues = [NSMutableArray arrayWithCapacity:[venueDicts count]];
					
					for (NSDictionary *dictionary in venueDicts) {
						VNKVenue *venue = [[VNKVenue alloc] initWithDictionary:dictionary service:self.service];
						[venues addObject:venue];
					}
					
					handler(venues, nil);
				}
				else {
					NSMutableDictionary *userInfo = [NSMutableDictionary dictionary];
					
					userInfo[NSLocalizedDescriptionKey] = NSLocalizedString(@"Foursquare API Error", nil);
					
					NSString *errorMessage = result[@"meta"][@"errorMessage"];
					if (errorMessage != nil) {
						userInfo[NSLocalizedFailureReasonErrorKey] = errorMessage;
						userInfo[NSLocalizedRecoverySuggestionErrorKey] = errorMessage;
					}
					
					NSError *error = [NSError errorWithDomain:VNKVenueKitErrorDomain code:statusCode userInfo:userInfo];
					handler(nil, error);
				}
			}
			else {
				handler(nil, JSONError);
			}
		}
		else {
			handler(nil, error);
		}
		
		self.searching = NO;
	}];
}

- (void)cancel {
	self.searching = NO;
}

@end
